@@ -3,56 +3,48 @@ use std::{collections::VecDeque, path::PathBuf, time::Duration};
 use exitfailure::ExitFailure;
 
 use crate::{
+    cli::structures::RunCommand,
     constants::{
         CACHE_FOLDER, GEN_BINARY_FILE, QTEST_ERROR_FILE, QTEST_INPUT_FILE, QTEST_OUTPUT_FILE,
         TARGET_BINARY_FILE,
     },
     error::handle_error::throw_compiler_error_msg,
-    file_handler::{
-        file::{
-            can_run_language_or_error, copy_file, create_folder_or_error, file_exists_or_error,
-            get_filename_output, is_extension_supported_or_error, load_testcases_from_prefix,
-            remove_files, save_test_case_output,
-        },
-        path::get_root_path,
+    file_handler::file::{
+        can_run_language_or_error, copy_file, create_folder_or_error, file_exists_or_error,
+        get_filename_output, is_extension_supported_or_error, load_testcases_from_prefix,
+        remove_files, save_test_case_output,
     },
+    language::language_handler::get_language_handler,
     painter::style::{show_ran_successfully, show_runtime_error, show_time_limit_exceeded},
     runner::types::{is_compiled_error, is_runtime_error, is_time_limit_exceeded, Language},
-    util::lang::get_language_by_ext_default,
 };
 
-pub fn run(
-    target_file: PathBuf,
-    prefix: &str,
-    timeout: u32,
-    break_bad: bool,
-    save_out: bool,
-) -> Result<(), ExitFailure> {
+pub fn run(command: &RunCommand) -> Result<(), ExitFailure> {
     // Check if the CACHE_FOLDER folder is already created
     create_folder_or_error(CACHE_FOLDER)?;
 
     // verify that the target file exists
-    file_exists_or_error(target_file.to_str().unwrap(), "<target-file>")?;
+    file_exists_or_error(command.target_file.to_str().unwrap(), "<target-file>")?;
 
     // verify that the target file extension is supported
-    is_extension_supported_or_error(target_file.to_str().unwrap())?;
-
-    let root = &get_root_path()[..];
+    is_extension_supported_or_error(command.target_file.to_str().unwrap())?;
 
     // Get the language depending on the extension of the target_file
-    let any_target: Option<Box<dyn Language>> = get_language_by_ext_default(
-        root,
-        target_file,
-        TARGET_BINARY_FILE,
+    let target_file_lang = *get_language_handler(
+        &command
+            .target_file
+            .clone()
+            .into_os_string()
+            .into_string()
+            .unwrap()[..],
+        "<target-file>",
         QTEST_INPUT_FILE,
         QTEST_OUTPUT_FILE,
         QTEST_ERROR_FILE,
-    );
-    let any_target: Box<dyn Language> = any_target.unwrap();
-    let target_file_lang: &dyn Language = any_target.as_ref();
+    )?;
 
     // verify that the program to run the target file is installed
-    can_run_language_or_error(target_file_lang)?;
+    can_run_language_or_error(&target_file_lang)?;
 
     let can_compile_target = target_file_lang.build();
     if !can_compile_target {
@@ -60,7 +52,7 @@ pub fn run(
     }
 
     let mut cases: VecDeque<PathBuf> = VecDeque::new();
-    load_testcases_from_prefix(&mut cases, prefix)?;
+    load_testcases_from_prefix(&mut cases, &command.prefix[..])?;
 
     let mut test_number: u32 = 0;
 
@@ -73,7 +65,7 @@ pub fn run(
         let case = cases.pop_front().unwrap();
         copy_file(case.to_str().unwrap(), QTEST_INPUT_FILE)?;
 
-        let response_target = target_file_lang.execute(timeout as u32, test_number);
+        let response_target = target_file_lang.execute(command.timeout as u32, test_number);
         let time_target: Duration = response_target.time;
         let mills_target = time_target.as_millis();
 
@@ -81,7 +73,7 @@ pub fn run(
             show_runtime_error(test_number, mills_target as u32);
 
             // check if the tle_breck flag is high
-            if break_bad {
+            if command.break_bad {
                 // remove input, output and error files
                 remove_files(vec![
                     QTEST_INPUT_FILE,
@@ -98,12 +90,12 @@ pub fn run(
             return throw_compiler_error_msg("target", "<target-file>");
         }
 
-        if time_target >= Duration::from_millis(timeout as u64)
+        if time_target >= Duration::from_millis(command.timeout as u64)
             || is_time_limit_exceeded(&response_target.status)
         {
-            show_time_limit_exceeded(test_number, timeout);
+            show_time_limit_exceeded(test_number, command.timeout);
             // check if the tle_breck flag is high
-            if break_bad {
+            if command.break_bad {
                 // remove input, output and error files
                 remove_files(vec![
                     QTEST_INPUT_FILE,
@@ -116,9 +108,11 @@ pub fn run(
                 return Ok(());
             }
         } else {
-            if save_out {
-                let file_name =
-                    &get_filename_output(prefix, case.file_name().unwrap().to_str().unwrap())[..];
+            if command.save_out {
+                let file_name = &get_filename_output(
+                    &command.prefix[..],
+                    case.file_name().unwrap().to_str().unwrap(),
+                )[..];
 
                 // save testcase
                 save_test_case_output(file_name, QTEST_OUTPUT_FILE);
